@@ -102,10 +102,10 @@ class behat_course extends behat_base {
             // Adding a forced wait until editors are loaded as otherwise selenium sometimes tries clicks on the
             // format field when the editor is being rendered and the click misses the field coordinates.
             $steps[] = new Given('I expand all fieldsets');
-            $steps[] = new Given('I select "' . $formatvalue . '" from "' . $formatfield . '"');
-            $steps[] = new Given('I fill the moodle form with:', $table);
+            $steps[] = new Given('I set the field "' . $formatfield . '" to "' . $formatvalue . '"');
+            $steps[] = new Given('I set the following fields to these values:', $table);
         } else {
-            $steps[] = new Given('I fill the moodle form with:', $table);
+            $steps[] = new Given('I set the following fields to these values:', $table);
         }
 
         $steps[] = new Given('I press "' . get_string('savechanges') . '"');
@@ -141,7 +141,7 @@ class behat_course extends behat_base {
 
         return array(
             new Given('I add a "' . $this->escape($activity) . '" to section "' . $this->escape($section) . '"'),
-            new Given('I fill the moodle form with:', $data),
+            new Given('I set the following fields to these values:', $data),
             new Given('I press "' . get_string('savechangesandreturntocourse') . '"')
         );
     }
@@ -288,7 +288,7 @@ class behat_course extends behat_base {
 
         return array(
             new Given('I edit the section "' . $sectionnumber . '"'),
-            new Given('I fill the moodle form with:', $data),
+            new Given('I set the following fields to these values:', $data),
             new Given('I press "' . get_string('savechanges') . '"')
         );
     }
@@ -498,7 +498,7 @@ class behat_course extends behat_base {
 
         if ($this->is_course_editor()) {
 
-            // The activity should exists.
+            // The activity should exist.
             $activitynode = $this->get_activity_node($activityname);
 
             // Should be hidden.
@@ -511,7 +511,7 @@ class behat_course extends behat_base {
 
         } else {
 
-            // It should not exists at all.
+            // It should not exist at all.
             try {
                 $this->find_link($activityname);
                 throw new ExpectationException('The "' . $activityname . '" should not appear');
@@ -577,7 +577,7 @@ class behat_course extends behat_base {
         $activity = $this->escape($activityname);
         return array(
             new Given('I click on "' . get_string('edittitle') . '" "link" in the "' . $activity .'" activity'),
-            new Given('I fill in "title" with "' . $this->escape($newactivityname) . chr(10) . '"')
+            new Given('I set the field "title" to "' . $this->escape($newactivityname) . chr(10) . '"')
         );
     }
 
@@ -635,7 +635,6 @@ class behat_course extends behat_base {
      * @Then /^"(?P<activity_name_string>(?:[^"]|\\")*)" actions menu should be open$/
      * @throws DriverException The step is not available when Javascript is disabled
      * @param string $activityname
-     * @return Given
      */
     public function actions_menu_should_be_open($activityname) {
 
@@ -703,26 +702,20 @@ class behat_course extends behat_base {
 
         $deletestring = get_string('delete');
 
+        $steps = array(
+            new Given('I click on "' . $this->escape($deletestring) . '" "link" in the "' . $this->escape($activityname) . '" activity')
+        );
+
         // JS enabled.
         // Not using chain steps here because the exceptions catcher have problems detecting
         // JS modal windows and avoiding interacting them at the same time.
         if ($this->running_javascript()) {
-
-            $element = $this->get_activity_element($deletestring, 'link', $activityname);
-            $element->click();
-
-            $this->getSession()->getDriver()->getWebDriverSession()->accept_alert();
-
+            $steps[] = new Given('I click on "' . get_string('yes') . '" "button" in the "Confirm" "dialogue"');
         } else {
-
-            // With JS disabled.
-            $steps = array(
-                new Given('I click on "' . $this->escape($deletestring) . '" "link" in the "' . $this->escape($activityname) . '" activity'),
-                new Given('I press "' . get_string('yes') . '"')
-            );
-
-            return $steps;
+            $steps[] = new Given('I press "' . get_string('yes') . '"');
         }
+
+        return $steps;
     }
 
     /**
@@ -790,7 +783,7 @@ class behat_course extends behat_base {
             $steps[] = new Given('I press "' . get_string('continue') .'"');
             $steps[] = new Given('I press "' . get_string('duplicatecontedit') . '"');
         }
-        $steps[] = new Given('I fill the moodle form with:', $data);
+        $steps[] = new Given('I set the following fields to these values:', $data);
         $steps[] = new Given('I press "' . get_string('savechangesandreturntocourse') . '"');
         return $steps;
     }
@@ -824,7 +817,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on the specified element of the activity. You should be in the course page with editing mode turned on.
      *
-     * @Given /^I click on "(?P<element_string>(?:[^"]|\\")*)" "(?P<selector_string>[^"]*)" in the "(?P<activity_name_string>[^"]*)" activity$/
+     * @Given /^I click on "(?P<element_string>(?:[^"]|\\")*)" "(?P<selector_string>(?:[^"]|\\")*)" in the "(?P<activity_name_string>(?:[^"]|\\")*)" activity$/
      * @param string $element
      * @param string $selectortype
      * @param string $activityname
@@ -1107,7 +1100,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on a category in the management interface.
      *
-     * @Given /^I click on category "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I click on category "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_click_on_category_in_the_management_interface($name) {
@@ -1118,7 +1111,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on a course in the management interface.
      *
-     * @Given /^I click on course "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I click on course "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_click_on_course_in_the_management_interface($name) {
@@ -1129,7 +1122,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on a category checkbox in the management interface, if not checked.
      *
-     * @Given /^I select category "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I select category "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_select_category_in_the_management_interface($name) {
@@ -1143,7 +1136,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on a category checkbox in the management interface, if checked.
      *
-     * @Given /^I unselect category "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I unselect category "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_unselect_category_in_the_management_interface($name) {
@@ -1157,7 +1150,7 @@ class behat_course extends behat_base {
     /**
      * Clicks course checkbox in the management interface, if not checked.
      *
-     * @Given /^I select course "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I select course "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_select_course_in_the_management_interface($name) {
@@ -1171,7 +1164,7 @@ class behat_course extends behat_base {
     /**
      * Clicks course checkbox in the management interface, if checked.
      *
-     * @Given /^I unselect course "(?P<name>[^"]*)" in the management interface$/
+     * @Given /^I unselect course "(?P<name_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $name
      */
     public function i_unselect_course_in_the_management_interface($name) {
@@ -1185,14 +1178,14 @@ class behat_course extends behat_base {
     /**
      * Move selected categories to top level in the management interface.
      *
-     * @Given /^I move category "(?P<name>[^"]*)" to top level in the management interface$/
+     * @Given /^I move category "(?P<name_string>(?:[^"]|\\")*)" to top level in the management interface$/
      * @param string $name
      * @return Given[]
      */
     public function i_move_category_to_top_level_in_the_management_interface($name) {
         $this->i_select_category_in_the_management_interface($name);
         return array(
-            new Given('I select "' .  coursecat::get(0)->get_formatted_name() . '" from "menumovecategoriesto"'),
+            new Given('I set the field "menumovecategoriesto" to "' .  coursecat::get(0)->get_formatted_name() . '"'),
             new Given('I press "bulkmovecategories"'),
         );
     }
@@ -1200,7 +1193,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a category is a subcategory of specific category.
      *
-     * @Given /^I should see category "(?P<subcatidnumber>[^"]*)" as subcategory of "(?P<catidnumber>[^"]*)" in the management interface$/
+     * @Given /^I should see category "(?P<subcatidnumber_string>(?:[^"]|\\")*)" as subcategory of "(?P<catidnumber_string>(?:[^"]|\\")*)" in the management interface$/
      * @throws ExpectationException
      * @param string $subcatidnumber
      * @param string $catidnumber
@@ -1216,7 +1209,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a category is not a subcategory of specific category.
      *
-     * @Given /^I should not see category "(?P<subcatidnumber>[^"]*)" as subcategory of "(?P<catidnumber>[^"]*)" in the management interface$/
+     * @Given /^I should not see category "(?P<subcatidnumber_string>(?:[^"]|\\")*)" as subcategory of "(?P<catidnumber_string>(?:[^"]|\\")*)" in the management interface$/
      * @throws ExpectationException
      * @param string $subcatidnumber
      * @param string $catidnumber
@@ -1234,7 +1227,7 @@ class behat_course extends behat_base {
     /**
      * Click to expand a category revealing its sub categories within the management UI.
      *
-     * @Given /^I click to expand category "(?P<idnumber>[^"]*)" in the management interface$/
+     * @Given /^I click to expand category "(?P<idnumber_string>(?:[^"]|\\")*)" in the management interface$/
      * @param string $idnumber
      */
     public function i_click_to_expand_category_in_the_management_interface($idnumber) {
@@ -1247,7 +1240,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a category within the management interface is visible.
      *
-     * @Given /^category in management listing should be visible "(?P<idnumber>[^"]*)"$/
+     * @Given /^category in management listing should be visible "(?P<idnumber_string>(?:[^"]|\\")*)"$/
      * @param string $idnumber
      */
     public function category_in_management_listing_should_be_visible($idnumber) {
@@ -1260,7 +1253,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a category within the management interface is dimmed.
      *
-     * @Given /^category in management listing should be dimmed "(?P<idnumber>[^"]*)"$/
+     * @Given /^category in management listing should be dimmed "(?P<idnumber_string>(?:[^"]|\\")*)"$/
      * @param string $idnumber
      */
     public function category_in_management_listing_should_be_dimmed($idnumber) {
@@ -1273,7 +1266,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a course within the management interface is visible.
      *
-     * @Given /^course in management listing should be visible "(?P<idnumber>[^"]*)"$/
+     * @Given /^course in management listing should be visible "(?P<idnumber_string>(?:[^"]|\\")*)"$/
      * @param string $idnumber
      */
     public function course_in_management_listing_should_be_visible($idnumber) {
@@ -1286,7 +1279,7 @@ class behat_course extends behat_base {
     /**
      * Checks that a course within the management interface is dimmed.
      *
-     * @Given /^course in management listing should be dimmed "(?P<idnumber>[^"]*)"$/
+     * @Given /^course in management listing should be dimmed "(?P<idnumber_string>(?:[^"]|\\")*)"$/
      * @param string $idnumber
      */
     public function course_in_management_listing_should_be_dimmed($idnumber) {
@@ -1301,7 +1294,7 @@ class behat_course extends behat_base {
      *
      * If it was visible it will be hidden. If it is hidden it will be made visible.
      *
-     * @Given /^I toggle visibility of course "(?P<idnumber>[^"]*)" in management listing$/
+     * @Given /^I toggle visibility of course "(?P<idnumber_string>(?:[^"]|\\")*)" in management listing$/
      * @param string $idnumber
      */
     public function i_toggle_visibility_of_course_in_management_listing($idnumber) {
@@ -1322,7 +1315,7 @@ class behat_course extends behat_base {
      *
      * If it was visible it will be hidden. If it is hidden it will be made visible.
      *
-     * @Given /^I toggle visibility of category "(?P<idnumber>[^"]*)" in management listing$/
+     * @Given /^I toggle visibility of category "(?P<idnumber_string>(?:[^"]|\\")*)" in management listing$/
      */
     public function i_toggle_visibility_of_category_in_management_listing($idnumber) {
         $id = $this->get_category_id($idnumber);
@@ -1340,7 +1333,7 @@ class behat_course extends behat_base {
     /**
      * Moves a category displayed in the management interface up or down one place.
      *
-     * @Given /^I click to move category "(?P<idnumber>[^"]*)" (?P<direction>up|down) one$/
+     * @Given /^I click to move category "(?P<idnumber_string>(?:[^"]|\\")*)" (?P<direction>up|down) one$/
      *
      * @param string $idnumber The category idnumber
      * @param string $direction The direction to move in, either up or down
@@ -1353,7 +1346,7 @@ class behat_course extends behat_base {
     /**
      * Moves a course displayed in the management interface up or down one place.
      *
-     * @Given /^I click to move course "(?P<idnumber>[^"]*)" (?P<direction>up|down) one$/
+     * @Given /^I click to move course "(?P<idnumber_string>(?:[^"]|\\")*)" (?P<direction>up|down) one$/
      *
      * @param string $idnumber The course idnumber
      * @param string $direction The direction to move in, either up or down
@@ -1403,7 +1396,7 @@ class behat_course extends behat_base {
     /**
      * Check that one course appears before another in the course category management listings.
      *
-     * @Given /^I should see course listing "(?P<preceedingcourse>[^"]*)" before "(?P<followingcourse>[^"]*)"$/
+     * @Given /^I should see course listing "(?P<preceedingcourse_string>(?:[^"]|\\")*)" before "(?P<followingcourse_string>(?:[^"]|\\")*)"$/
      *
      * @param string $preceedingcourse The first course to find
      * @param string $followingcourse The second course to find (should be AFTER the first course)
@@ -1420,7 +1413,7 @@ class behat_course extends behat_base {
     /**
      * Check that one category appears before another in the course category management listings.
      *
-     * @Given /^I should see category listing "(?P<preceedingcategory>[^"]*)" before "(?P<followingcategory>[^"]*)"$/
+     * @Given /^I should see category listing "(?P<preceedingcategory_string>(?:[^"]|\\")*)" before "(?P<followingcategory_string>(?:[^"]|\\")*)"$/
      *
      * @param string $preceedingcategory The first category to find
      * @param string $followingcategory The second category to find (should be after the first category)
@@ -1437,7 +1430,7 @@ class behat_course extends behat_base {
     /**
      * Checks that we are on the course management page that we expect to be on and that no course has been selected.
      *
-     * @Given /^I should see the "(?P<mode>[^"]*)" management page$/
+     * @Given /^I should see the "(?P<mode_string>(?:[^"]|\\")*)" management page$/
      * @param string $mode The mode to expected. One of 'Courses', 'Course categories' or 'Course categories and courses'
      * @return Given[]
      */
@@ -1447,41 +1440,41 @@ class behat_course extends behat_base {
         );
         switch ($mode) {
             case "Courses":
-                $return[] = new Given('"#category-listing" "css_element" should not exists');
-                $return[] = new Given('"#course-listing" "css_element" should exists');
+                $return[] = new Given('"#category-listing" "css_element" should not exist');
+                $return[] = new Given('"#course-listing" "css_element" should exist');
                 break;
             case "Course categories":
-                $return[] = new Given('"#category-listing" "css_element" should exists');
-                $return[] = new Given('"#course-listing" "css_element" should not exists');
+                $return[] = new Given('"#category-listing" "css_element" should exist');
+                $return[] = new Given('"#course-listing" "css_element" should not exist');
                 break;
             case "Courses categories and courses":
             default:
-                $return[] = new Given('"#category-listing" "css_element" should exists');
-                $return[] = new Given('"#course-listing" "css_element" should exists');
+                $return[] = new Given('"#category-listing" "css_element" should exist');
+                $return[] = new Given('"#course-listing" "css_element" should exist');
                 break;
         }
-        $return[] = new Given('"#course-detail" "css_element" should not exists');
+        $return[] = new Given('"#course-detail" "css_element" should not exist');
         return $return;
     }
 
     /**
      * Checks that we are on the course management page that we expect to be on and that a course has been selected.
      *
-     * @Given /^I should see the "(?P<mode>[^"]*)" management page with a course selected$/
+     * @Given /^I should see the "(?P<mode_string>(?:[^"]|\\")*)" management page with a course selected$/
      * @param string $mode The mode to expected. One of 'Courses', 'Course categories' or 'Course categories and courses'
      * @return Given[]
      */
     public function i_should_see_the_courses_management_page_with_a_course_selected($mode) {
         $return = $this->i_should_see_the_courses_management_page($mode);
         array_pop($return);
-        $return[] = new Given('"#course-detail" "css_element" should exists');
+        $return[] = new Given('"#course-detail" "css_element" should exist');
         return $return;
     }
 
     /**
      * Locates a course in the course category management interface and then triggers an action for it.
      *
-     * @Given /^I click on "(?P<action>[^"]*)" action for "(?P<name>[^"]*)" in management course listing$/
+     * @Given /^I click on "(?P<action_string>(?:[^"]|\\")*)" action for "(?P<name_string>(?:[^"]|\\")*)" in management course listing$/
      *
      * @param string $action The action to take. One of
      * @param string $name The name of the course as it is displayed in the management interface.
@@ -1494,7 +1487,7 @@ class behat_course extends behat_base {
     /**
      * Locates a category in the course category management interface and then triggers an action for it.
      *
-     * @Given /^I click on "(?P<action>[^"]*)" action for "(?P<name>[^"]*)" in management category listing$/
+     * @Given /^I click on "(?P<action_string>(?:[^"]|\\")*)" action for "(?P<name_string>(?:[^"]|\\")*)" in management category listing$/
      *
      * @param string $action The action to take. One of
      * @param string $name The name of the category as it is displayed in the management interface.
@@ -1555,7 +1548,7 @@ class behat_course extends behat_base {
     /**
      * Clicks on a category in the management interface.
      *
-     * @Given /^I click on "([^"]*)" category in the management category listing$/
+     * @Given /^I click on "(?P<categoryname_string>(?:[^"]|\\")*)" category in the management category listing$/
      * @param string $name The name of the category to click.
      */
     public function i_click_on_category_in_the_management_category_listing($name) {
