@@ -1040,14 +1040,14 @@ class completion_info {
         $coursecontext = $cmcontext->get_parent_context();
 
         // Trigger an event for course module completion changed.
-        $event = \core\event\course_module_completion_updated::create(
-            array('objectid' => $data->id,
-                'userid' => $USER->id,
-                'context' => $cmcontext,
-                'courseid' => $coursecontext->instanceid,
-                'other' => array('relateduserid' => $data->userid)
-                )
-            );
+        $event = \core\event\course_module_completion_updated::create(array(
+            'objectid' => $data->id,
+            'context' => $cmcontext,
+            'relateduserid' => $data->userid,
+            'other' => array(
+                'relateduserid' => $data->userid
+            )
+        ));
         $event->add_record_snapshot('course_modules_completion', $data);
         $event->trigger();
 
@@ -1278,9 +1278,12 @@ class completion_info {
      * @return int Completion state e.g. COMPLETION_INCOMPLETE
      */
     public static function internal_get_grade_state($item, $grade) {
-        if (!$grade) {
+        // If no grade is supplied or the grade doesn't have an actual value, then
+        // this is not complete.
+        if (!$grade || (is_null($grade->finalgrade) && is_null($grade->rawgrade))) {
             return COMPLETION_INCOMPLETE;
         }
+
         // Conditions to show pass/fail:
         // a) Grade has pass mark (default is 0.00000 which is boolean true so be careful)
         // b) Grade is visible (neither hidden nor hidden-until)
