@@ -35,22 +35,7 @@ var LOGNS = 'moodle-core-dock',
     BLOCK,
     DOCKEDITEM;
 
-/**
- * Core namespace.
- *
- * @static
- * @namespace M
- * @class core
- */
 M.core = M.core || {};
-
-/**
- * Dock namespace.
- *
- * @static
- * @namespace M.core
- * @class dock
- */
 M.core.dock = M.core.dock || {};
 
 /**
@@ -259,8 +244,8 @@ M.core.dock.notifyBlockChange = function(instanceid) {
  * @namespace M.core.dock
  * @class Dock
  * @constructor
- * @extends Y.Base
- * @uses Y.EventTarget
+ * @extends Base
+ * @uses EventTarget
  */
 DOCK = function() {
     DOCK.superclass.constructor.apply(this, arguments);
@@ -980,7 +965,7 @@ DOCK.prototype = {
         }
 
         if (this.get('position') === 'right') {
-            panel.get('node').setStyle('left', '-' + panel.get('node').get('offsetWidth') + 'px');
+            panel.get('node').setStyle('left', -panel.get('offsetWidth')+'px');
 
         } else if (this.get('position') === 'top') {
             dockx = this.get('dockNode').getX();
