@@ -21,8 +21,7 @@
  *
  * Included by import.ph
  *
- * @package    mod
- * @subpackage lesson
+ * @package mod_lesson
  * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  **/
@@ -283,8 +282,9 @@ function lesson_save_question_options($question, $lesson) {
 class qformat_default {
 
     var $displayerrors = true;
-    var $category = NULL;
+    var $category = null;
     var $questionids = array();
+    protected $importcontext = null;
     var $qtypeconvert = array('numerical'   => LESSON_PAGE_NUMERICAL,
                                'multichoice' => LESSON_PAGE_MULTICHOICE,
                                'truefalse'   => LESSON_PAGE_TRUEFALSE,
@@ -297,6 +297,30 @@ class qformat_default {
         return false;
     }
 
+    function set_importcontext($context) {
+        $this->importcontext = $context;
+    }
+
+    /**
+     * Handle parsing error
+     *
+     * @param string $message information about error
+     * @param string $text imported text that triggered the error
+     * @param string $questionname imported question name
+     */
+    protected function error($message, $text='', $questionname='') {
+        $importerrorquestion = get_string('importerrorquestion', 'question');
+
+        echo "<div class=\"importerror\">\n";
+        echo "<strong>$importerrorquestion $questionname</strong>";
+        if (!empty($text)) {
+            $text = s($text);
+            echo "<blockquote>$text</blockquote>\n";
+        }
+        echo "<strong>$message</strong>\n";
+        echo "</div>";
+    }
+
     function importpreprocess() {
         // Does any pre-processing that may be desired
         return true;
@@ -410,6 +434,15 @@ class qformat_default {
 
                     $this->questionids[] = $question->id;
 
+                    // Import images in question text.
+                    if (isset($question->questiontextitemid)) {
+                        $questiontext = file_save_draft_area_files($question->questiontextitemid,
+                                $this->importcontext->id, 'mod_lesson', 'page_contents', $newpageid,
+                                null , $question->questiontext);
+                        // Update content with recoded urls.
+                        $DB->set_field("lesson_pages", "contents", $questiontext, array("id" => $newpageid));
+                    }
+
                     // Now to save all the answers and type-specific options
 
                     $question->lessonid = $lesson->id; // needed for foreign key
@@ -524,7 +557,7 @@ class qformat_default {
 
         echo "<p>This flash question format has not yet been completed!</p>";
 
-        return NULL;
+        return null;
     }
 
     /**
@@ -551,7 +584,7 @@ class qformat_default {
         $name = clean_param($name, PARAM_TEXT); // Matches what the question editing form does.
         $name = trim($name);
         $trimlength = 251;
-        while (textlib::strlen($name) > 255 && $trimlength > 0) {
+        while (core_text::strlen($name) > 255 && $trimlength > 0) {
             $name = shorten_text($name, $trimlength);
             $trimlength -= 10;
         }
@@ -603,7 +636,12 @@ class qformat_default {
     protected function format_question_text($question) {
         $formatoptions = new stdClass();
         $formatoptions->noclean = true;
-        return html_to_text(format_text($question->questiontext,
+        // The html_to_text call strips out all URLs, but format_text complains
+        // if it finds @@PLUGINFILE@@ tokens. So, we need to replace
+        // @@PLUGINFILE@@ with a real URL, but it doesn't matter what.
+        // We use http://example.com/.
+        $text = str_replace('@@PLUGINFILE@@/', 'http://example.com/', $question->questiontext);
+        return html_to_text(format_text($text,
                 $question->questiontextformat, $formatoptions), 0, false);
     }
 
@@ -640,8 +678,8 @@ class qformat_based_on_xml extends qformat_default {
             "&#8212;" => "-",
         );
         $str = strtr($str, $html_code_list);
-        // Use textlib entities_to_utf8 function to convert only numerical entities.
-        $str = textlib::entities_to_utf8($str, false);
+        // Use core_text entities_to_utf8 function to convert only numerical entities.
+        $str = core_text::entities_to_utf8($str, false);
         return $str;
     }
 
