@@ -23,8 +23,7 @@
  * the module-indpendent code for handling questions and which in turn
  * initialises all the questiontype classes.
  *
- * @package    mod
- * @subpackage quiz
+ * @package    mod_quiz
  * @copyright  1999 onwards Martin Dougiamas and others {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -110,10 +109,7 @@ function quiz_create_attempt(quiz $quizobj, $attemptnumber, $lastattempt, $timen
         $attempt->quiz = $quiz->id;
         $attempt->userid = $userid;
         $attempt->preview = 0;
-        $attempt->layout = quiz_clean_layout($quiz->questions, true);
-        if ($quiz->shufflequestions) {
-            $attempt->layout = quiz_repaginate($attempt->layout, $quiz->questionsperpage, true);
-        }
+        $attempt->layout = '';
     } else {
         // Build on last attempt.
         if (empty($lastattempt)) {
@@ -127,6 +123,8 @@ function quiz_create_attempt(quiz $quizobj, $attemptnumber, $lastattempt, $timen
     $attempt->timefinish = 0;
     $attempt->timemodified = $timenow;
     $attempt->state = quiz_attempt::IN_PROGRESS;
+    $attempt->currentpage = 0;
+    $attempt->sumgrades = null;
 
     // If this is a preview, mark it as such.
     if ($ispreview) {
@@ -165,9 +163,8 @@ function quiz_start_new_attempt($quizobj, $quba, $attempt, $attemptnumber, $time
     $quizobj->load_questions();
 
     // Add them all to the $quba.
-    $idstoslots = array();
     $questionsinuse = array_keys($quizobj->get_questions());
-    foreach ($quizobj->get_questions() as $i => $questiondata) {
+    foreach ($quizobj->get_questions() as $questiondata) {
         if ($questiondata->qtype != 'random') {
             if (!$quizobj->get_quiz()->shuffleanswers) {
                 $questiondata->options->shuffleanswers = false;
@@ -189,7 +186,7 @@ function quiz_start_new_attempt($quizobj, $quba, $attempt, $attemptnumber, $time
             }
         }
 
-        $idstoslots[$i] = $quba->add_question($question, $questiondata->maxmark);
+        $quba->add_question($question, $questiondata->maxmark);
         $questionsinuse[] = $question->id;
     }
 
@@ -211,16 +208,36 @@ function quiz_start_new_attempt($quizobj, $quba, $attempt, $attemptnumber, $time
 
     $quba->start_all_questions($variantstrategy, $timenow);
 
-    // Update attempt layout.
-    $newlayout = array();
-    foreach (explode(',', $attempt->layout) as $qid) {
-        if ($qid != 0) {
-            $newlayout[] = $idstoslots[$qid];
-        } else {
-            $newlayout[] = 0;
+    // Work out the attempt layout.
+    $layout = array();
+    if ($quizobj->get_quiz()->shufflequestions) {
+        $slots = $quba->get_slots();
+        shuffle($slots);
+
+        $questionsonthispage = 0;
+        foreach ($slots as $slot) {
+            if ($questionsonthispage && $questionsonthispage == $quizobj->get_quiz()->questionsperpage) {
+                $layout[] = 0;
+                $questionsonthispage = 0;
+            }
+            $layout[] = $slot;
+            $questionsonthispage += 1;
+        }
+
+    } else {
+        $currentpage = null;
+        foreach ($quizobj->get_questions() as $slot) {
+            if ($currentpage !== null && $slot->page != $currentpage) {
+                $layout[] = 0;
+            }
+            $layout[] = $slot->slot;
+            $currentpage = $slot->page;
         }
     }
-    $attempt->layout = implode(',', $newlayout);
+
+    $layout[] = 0;
+    $attempt->layout = implode(',', $layout);
+
     return $attempt;
 }
 
@@ -272,14 +289,29 @@ function quiz_attempt_save_started($quizobj, $quba, $attempt) {
     question_engine::save_questions_usage_by_activity($quba);
     $attempt->uniqueid = $quba->get_id();
     $attempt->id = $DB->insert_record('quiz_attempts', $attempt);
-    // Log the new attempt.
+
+    // Params used by the events below.
+    $params = array(
+        'objectid' => $attempt->id,
+        'relateduserid' => $attempt->userid,
+        'courseid' => $quizobj->get_courseid(),
+        'context' => $quizobj->get_context()
+    );
+    // Decide which event we are using.
     if ($attempt->preview) {
-        add_to_log($quizobj->get_courseid(), 'quiz', 'preview', 'view.php?id='.$quizobj->get_cmid(),
-                   $quizobj->get_quizid(), $quizobj->get_cmid());
+        $params['other'] = array(
+            'quizid' => $quizobj->get_quizid()
+        );
+        $event = \mod_quiz\event\attempt_preview_started::create($params);
     } else {
-        add_to_log($quizobj->get_courseid(), 'quiz', 'attempt', 'review.php?attempt='.$attempt->id,
-                   $quizobj->get_quizid(), $quizobj->get_cmid());
+        $event = \mod_quiz\event\attempt_started::create($params);
+
     }
+
+    // Trigger the event.
+    $event->add_record_snapshot('quiz', $quizobj->get_quiz());
+    $event->trigger();
+
     return $attempt;
 }
 
@@ -340,9 +372,27 @@ function quiz_delete_attempt($attempt, $quiz) {
         return;
     }
 
+    if (!isset($quiz->cmid)) {
+        $cm = get_coursemodule_from_instance('quiz', $quiz->id, $quiz->course);
+        $quiz->cmid = $cm->id;
+    }
+
     question_engine::delete_questions_usage_by_activity($attempt->uniqueid);
     $DB->delete_records('quiz_attempts', array('id' => $attempt->id));
 
+    // Log the deletion of the attempt.
+    $params = array(
+        'objectid' => $attempt->id,
+        'relateduserid' => $attempt->userid,
+        'context' => context_module::instance($quiz->cmid),
+        'other' => array(
+            'quizid' => $quiz->id
+        )
+    );
+    $event = \mod_quiz\event\attempt_deleted::create($params);
+    $event->add_record_snapshot('quiz_attempts', $attempt);
+    $event->trigger();
+
     // Search quiz_attempts for other instances by this user.
     // If none, then delete record for this quiz, this user from quiz_grades
     // else recalculate best grade.
@@ -386,127 +436,35 @@ function quiz_has_attempts($quizid) {
 // Functions to do with quiz layout and pages //////////////////////////////////
 
 /**
- * Returns a comma separated list of question ids for the quiz
- *
- * @param string $layout The string representing the quiz layout. Each page is
- *      represented as a comma separated list of question ids and 0 indicating
- *      page breaks. So 5,2,0,3,0 means questions 5 and 2 on page 1 and question
- *      3 on page 2
- * @return string comma separated list of question ids, without page breaks.
+ * Repaginate the questions in a quiz
+ * @param int $quizid the id of the quiz to repaginate.
+ * @param int $slotsperpage number of items to put on each page. 0 means unlimited.
  */
-function quiz_questions_in_quiz($layout) {
-    $questions = str_replace(',0', '', quiz_clean_layout($layout, true));
-    if ($questions === '0') {
-        return '';
-    } else {
-        return $questions;
-    }
-}
+function quiz_repaginate_questions($quizid, $slotsperpage) {
+    global $DB;
+    $trans = $DB->start_delegated_transaction();
 
-/**
- * Returns the number of pages in a quiz layout
- *
- * @param string $layout The string representing the quiz layout. Always ends in ,0
- * @return int The number of pages in the quiz.
- */
-function quiz_number_of_pages($layout) {
-    return substr_count(',' . $layout, ',0');
-}
+    $slots = $DB->get_records('quiz_slots', array('quizid' => $quizid),
+            'slot');
 
-/**
- * Returns the number of questions in the quiz layout
- *
- * @param string $layout the string representing the quiz layout.
- * @return int The number of questions in the quiz.
- */
-function quiz_number_of_questions_in_quiz($layout) {
-    $layout = quiz_questions_in_quiz(quiz_clean_layout($layout));
-    $count = substr_count($layout, ',');
-    if ($layout !== '') {
-        $count++;
-    }
-    return $count;
-}
-
-/**
- * Re-paginates the quiz layout
- *
- * @param string $layout  The string representing the quiz layout. If there is
- *      if there is any doubt about the quality of the input data, call
- *      quiz_clean_layout before you call this function.
- * @param int $perpage The number of questions per page
- * @param bool $shuffle Should the questions be reordered randomly?
- * @return string the new layout string
- */
-function quiz_repaginate($layout, $perpage, $shuffle = false) {
-    $questions = quiz_questions_in_quiz($layout);
-    if (!$questions) {
-        return '0';
-    }
-
-    $questions = explode(',', quiz_questions_in_quiz($layout));
-    if ($shuffle) {
-        shuffle($questions);
-    }
-
-    $onthispage = 0;
-    $layout = array();
-    foreach ($questions as $question) {
-        if ($perpage and $onthispage >= $perpage) {
-            $layout[] = 0;
-            $onthispage = 0;
+    $currentpage = 1;
+    $slotsonthispage = 0;
+    foreach ($slots as $slot) {
+        if ($slotsonthispage && $slotsonthispage == $slotsperpage) {
+            $currentpage += 1;
+            $slotsonthispage = 0;
         }
-        $layout[] = $question;
-        $onthispage += 1;
+        if ($slot->page != $currentpage) {
+            $DB->set_field('quiz_slots', 'page', $currentpage, array('id' => $slot->id));
+        }
+        $slotsonthispage += 1;
     }
 
-    $layout[] = 0;
-    return implode(',', $layout);
+    $trans->allow_commit();
 }
 
 // Functions to do with quiz grades ////////////////////////////////////////////
 
-/**
- * Creates an array of maximum grades for a quiz
- *
- * The grades are extracted from the quiz_question_instances table.
- * @param object $quiz The quiz settings.
- * @return array of grades indexed by question id. These are the maximum
- *      possible grades that students can achieve for each of the questions.
- */
-function quiz_get_all_question_grades($quiz) {
-    global $CFG, $DB;
-
-    $questionlist = quiz_questions_in_quiz($quiz->questions);
-    if (empty($questionlist)) {
-        return array();
-    }
-
-    $params = array($quiz->id);
-    $wheresql = '';
-    if (!is_null($questionlist)) {
-        list($usql, $question_params) = $DB->get_in_or_equal(explode(',', $questionlist));
-        $wheresql = " AND question $usql ";
-        $params = array_merge($params, $question_params);
-    }
-
-    $instances = $DB->get_records_sql("SELECT question, grade, id
-                                    FROM {quiz_question_instances}
-                                    WHERE quiz = ? $wheresql", $params);
-
-    $list = explode(",", $questionlist);
-    $grades = array();
-
-    foreach ($list as $qid) {
-        if (isset($instances[$qid])) {
-            $grades[$qid] = $instances[$qid]->grade;
-        } else {
-            $grades[$qid] = 1;
-        }
-    }
-    return $grades;
-}
-
 /**
  * Convert the raw grade stored in $attempt into a grade out of the maximum
  * grade for this quiz.
@@ -601,9 +559,9 @@ function quiz_update_sumgrades($quiz) {
 
     $sql = 'UPDATE {quiz}
             SET sumgrades = COALESCE((
-                SELECT SUM(grade)
-                FROM {quiz_question_instances}
-                WHERE quiz = {quiz}.id
+                SELECT SUM(maxmark)
+                FROM {quiz_slots}
+                WHERE quizid = {quiz}.id
             ), 0)
             WHERE id = ?';
     $DB->execute($sql, array($quiz->id));
@@ -1168,8 +1126,21 @@ function quiz_get_overdue_handling_options() {
 }
 
 /**
- * @param string $state one of the state constants like IN_PROGRESS.
- * @return string the human-readable state name.
+ * Get the choices for what size user picture to show.
+ * @return array string => lang string the options for whether to display the user's picture.
+ */
+function quiz_get_user_image_options() {
+    return array(
+        QUIZ_SHOWIMAGE_NONE  => get_string('shownoimage', 'quiz'),
+        QUIZ_SHOWIMAGE_SMALL => get_string('showsmallimage', 'quiz'),
+        QUIZ_SHOWIMAGE_LARGE => get_string('showlargeimage', 'quiz'),
+    );
+}
+
+/**
+ * Get the human-readable name for a quiz attempt state.
+ * @param string $state one of the state constants like {@link quiz_attempt::IN_PROGRESS}.
+ * @return string The lang string to describe that state.
  */
 function quiz_attempt_state_name($state) {
     switch ($state) {
@@ -1375,6 +1346,7 @@ function quiz_get_review_options($quiz, $attempt, $context) {
         $options->marks = question_display_options::MARK_AND_MAX;
         $options->feedback = question_display_options::VISIBLE;
         $options->numpartscorrect = question_display_options::VISIBLE;
+        $options->manualcomment = question_display_options::VISIBLE;
         $options->generalfeedback = question_display_options::VISIBLE;
         $options->rightanswer = question_display_options::VISIBLE;
         $options->overallfeedback = question_display_options::VISIBLE;
@@ -1424,73 +1396,6 @@ function quiz_get_combined_reviewoptions($quiz, $attempts) {
     return array($someoptions, $alloptions);
 }
 
-/**
- * Clean the question layout from various possible anomalies:
- * - Remove consecutive ","'s
- * - Remove duplicate question id's
- * - Remove extra "," from beginning and end
- * - Finally, add a ",0" in the end if there is none
- *
- * @param $string $layout the quiz layout to clean up, usually from $quiz->questions.
- * @param bool $removeemptypages If true, remove empty pages from the quiz. False by default.
- * @return $string the cleaned-up layout
- */
-function quiz_clean_layout($layout, $removeemptypages = false) {
-    // Remove repeated ','s. This can happen when a restore fails to find the right
-    // id to relink to.
-    $layout = preg_replace('/,{2,}/', ',', trim($layout, ','));
-
-    // Remove duplicate question ids.
-    $layout = explode(',', $layout);
-    $cleanerlayout = array();
-    $seen = array();
-    foreach ($layout as $item) {
-        if ($item == 0) {
-            $cleanerlayout[] = '0';
-        } else if (!in_array($item, $seen)) {
-            $cleanerlayout[] = $item;
-            $seen[] = $item;
-        }
-    }
-
-    if ($removeemptypages) {
-        // Avoid duplicate page breaks.
-        $layout = $cleanerlayout;
-        $cleanerlayout = array();
-        $stripfollowingbreaks = true; // Ensure breaks are stripped from the start.
-        foreach ($layout as $item) {
-            if ($stripfollowingbreaks && $item == 0) {
-                continue;
-            }
-            $cleanerlayout[] = $item;
-            $stripfollowingbreaks = $item == 0;
-        }
-    }
-
-    // Add a page break at the end if there is none.
-    if (end($cleanerlayout) !== '0') {
-        $cleanerlayout[] = '0';
-    }
-
-    return implode(',', $cleanerlayout);
-}
-
-/**
- * Get the slot for a question with a particular id.
- * @param object $quiz the quiz settings.
- * @param int $questionid the of a question in the quiz.
- * @return int the corresponding slot. Null if the question is not in the quiz.
- */
-function quiz_get_slot_for_question($quiz, $questionid) {
-    $questionids = quiz_questions_in_quiz($quiz->questions);
-    foreach (explode(',', $questionids) as $key => $id) {
-        if ($id == $questionid) {
-            return $key + 1;
-        }
-    }
-    return null;
-}
-
 // Functions for sending notification messages /////////////////////////////////
 
 /**
@@ -1836,7 +1741,9 @@ function quiz_process_group_deleted_in_course($courseid) {
               FROM {quiz_overrides} o
               JOIN {quiz} quiz ON quiz.id = o.quiz
          LEFT JOIN {groups} grp ON grp.id = o.groupid
-             WHERE quiz.course = :courseid AND grp.id IS NULL";
+             WHERE quiz.course = :courseid
+               AND o.groupid IS NOT NULL
+               AND grp.id IS NULL";
     $params = array('courseid' => $courseid);
     $records = $DB->get_records_sql_menu($sql, $params);
     if (!$records) {
@@ -1936,6 +1843,7 @@ class mod_quiz_display_options extends question_display_options {
         $options->overallfeedback = self::extract($quiz->reviewoverallfeedback, $when);
 
         $options->numpartscorrect = $options->feedback;
+        $options->manualcomment = $options->feedback;
 
         if ($quiz->questiondecimalpoints != -1) {
             $options->markdp = $quiz->questiondecimalpoints;
